use btproto::protocol;
use btrun::{ActorName, CallMsg};
use btsector::FileId; 
use btlib::Result;
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize)]
pub struct Open {
    id: FileId,
}

impl CallMsg for Open {
    type Reply = ActorName;
}
protocol! {
    let name = FsProtocol;
    let states = [
        ServerInit, Listening,
        Client,
        FileInit, Opened,
        FileHandle,
    ];
    ServerInit?Activate -> Listening;

    Client -> Client, >service(Listening)!Query;
    Listening?Query -> Listening, >Client!Query::Reply;
    Client?Query::Reply -> Client;

    Client -> Client, >service(Listening)!Open;
    Listening?Open -> Listening, FileInit, >Client!Open::Reply[Opened], FileInit!Open;
    Client?Open::Reply[Opened] -> Client, FileHandle[Opened];

    FileInit?Activate -> FileInit;
    FileInit?Open -> Opened;

    FileHandle[Opened] -> FileHandle[Opened], >Opened!FileOp;
    Opened?FileOp -> Opened, >FileHandle!FileOp::Reply;
    FileHandle?FileOp::Reply -> FileHandle;

    FileHandle[Opened] -> End, >Opened!Close;
    Opened?Close -> End;
}

#[derive(Serialize, Deserialize)]
pub struct Query;

impl CallMsg for Query {
    type Reply = ();
}

#[derive(Serialize, Deserialize)]
pub struct FileOp;

impl CallMsg for FileOp {
    type Reply = ();
}

#[derive(Serialize, Deserialize)]
pub struct Close;
