extern crate proc_macro;
use proc_macro::TokenStream;
use quote::ToTokens;
use syn::parse_macro_input;

mod parsing;
use parsing::Protocol;

mod error;
mod generation;
mod validation;

macro_rules! unwrap_or_compile_err {
    ($result:expr) => {
        match $result {
            Ok(value) => value,
            Err(err) => return err.into_compile_error().into(),
        }
    };
}

/// Generates types for the parties participating in a messaging protocol.
/// The grammar recognized by this macro is given below in the dialect of Extended Backus-Naur Form
/// recognized by the `llgen` tool:
///
/// ```ebnf
/// protocol : name_def states_def transition* ;
/// name_def : "let" "name" '=' Ident ';' ;
/// states_def : "let" "states" '=' ident_array ';' ;
/// ident_array : '[' Ident ( ',' Ident )* ','? ']' ;
/// transition : state ( '?' message )?  "->" states_list ( '>' dest_list )? ';' ;
/// state : Ident ident_array? ;
/// states_list : state ( ',' state )* ','? ;
/// dest_list : dest ( ',' dest )* ;
/// dest : dest_state '!' message
/// dest_state : ( "service" '(' Ident ')' ) | state ;
/// message : Ident ( "::" "Reply" )? ident_array? ;
/// ```
#[proc_macro]
pub fn protocol(input: TokenStream) -> TokenStream {
    let input = parse_macro_input!(input as Protocol);
    unwrap_or_compile_err!(input.validate());
    input.to_token_stream().into()
}
