//! Types which define the protocol used by the sector layer.

use btrun::CallMsg;
use btlib::{Result, crypto::merkle_stream::VariantMerkleTree, BlockMeta, Inode};
use btproto::protocol;

use serde::{Serialize, Deserialize};

#[derive(Serialize, Deserialize)]
pub struct SectorMsg {
    id: SectorId,
    op: SectorOperation,
}

impl CallMsg for SectorMsg {
    type Reply = SectorMsgReply;
}

#[derive(Serialize, Deserialize)]
pub struct SectorId {
    file: FileId,
    sector: SectorKind,
}

#[derive(Serialize, Deserialize)]
pub struct FileId {
    generation: u64,
    inode: Inode,
}

#[derive(Serialize, Deserialize)]
pub enum SectorKind {
    Meta,
    Merkle,
    Data(u64),
}

#[derive(Serialize, Deserialize)]
pub enum SectorOperation {
    Read,
    Write(WriteOperation),
}

#[derive(Serialize, Deserialize)]
pub enum WriteOperation {
    Meta(Box<BlockMeta>),
    Data {
        meta: Box<BlockMeta>,
        contents: Vec<u8>,
    },
}

#[derive(Serialize, Deserialize)]
pub enum SectorMsgReply {
    Meta(BlockMeta),
    Merkle(VariantMerkleTree),
    Data(Vec<u8>),
}

protocol! {
    let name = SectorProtocol;
    let states = [ServerInit, Listening, Client];
    ServerInit?Activate -> Listening;
    Client -> Client, >service(Listening)!SectorMsg;
    Listening?SectorMsg -> Listening, >Client!SectorMsg::Reply;
    Client?SectorMsg::Reply -> Client;
}
